

/** Light measurement.
 * This routines convert the amount of light into a binary number.
 * The only external device needed is a capacitor and the light
 * sensor which. Capacitor is from GND to input pin, light sensor
 * is from input pin to VCC.
 * The capacitor is first set to zero volts. The time that is needed
 * by the light sensor to load the capacitor is taken as the
 * light representing value.
 *
 * @author Clemens Helfmeier
 * @date 12.8.06
 */

#include "led.h"
#include "light.h"
#include <avr/io.h>

#include <util/delay.h>

#define LIGHT_PORT	PORTD
#define LIGHT_PIN	PIND
#define LIGHT_DIR	DDRD
#define LIGHT_BIT	(1<<2)


uint8_t light_cnt;

#define LIGHT_FINISHED	(1<<0)
uint8_t light_status;

void light_init( void ) {
	LIGHT_PORT &= ~LIGHT_BIT;
	light_start();
}

void light_timer( void ) {
	if (((LIGHT_PIN & LIGHT_BIT) == 0) && ((light_status & LIGHT_FINISHED) == 0)) {
		light_cnt++;
		if (light_cnt == 0) {
			light_cnt--;
			light_status |= LIGHT_FINISHED;
		}
	} else {
		// light measurement has finished
		light_status |= LIGHT_FINISHED;
	}
}

void light_start( void ) {
	light_cnt = 0;
	LIGHT_DIR |= LIGHT_BIT;
	_delay_us( 50 );
	LIGHT_DIR &= ~LIGHT_BIT;
	light_status &= ~LIGHT_FINISHED;
}

void light_slow_timer( void ) {
	if ((light_status & LIGHT_FINISHED) != 0) {
		// the light measurement has been finished.
		if (light_cnt > 0xC0)
			led_setLight( 50 );
		else
			led_setLight( 20000 );
	}
}
