
/** LED Watch
 * watch that recieves the time from a DCF77 module and
 * displays it on the attached LED displays. It will also
 * regulate the light of the LED display.
 *
 * @author Clemens Helfmeier
 * @date 12.08.06
 */


#include "led.h"
#include "bit_decoder.h"
#include "dcf77.h"
#include "light.h"

#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>

uint8_t Timer200ms;
uint8_t SyncTimer;
int8_t Timer1s;

int main( void ) {
	
	wdt_disable();
	
	led_init();
	bdec_init();
	light_init();

	/* Timer 0 has prescaler 64 and TOP of 250 and thus
	* results in exactly 625 Hz */
	TCCR0A = (1<<WGM01) | (0<<WGM00);
	TCCR0B = (0<<WGM02) | (0<<CS02) | (1<<CS01) | (1<<CS00);
	OCR0A = 249;
	TIMSK |= (1<<OCIE0A);

	sei();
	
	uint8_t dot = 0;
	while(1) {
		uint8_t i = bdec_getBit();
		if (i != bdec_EMPTY) {
			if (SyncTimer > 7) {
				dcf77_putSync();
			}
			SyncTimer = 0;
			if (i == bdec_ONE)
				dcf77_putBit( 1 );
			else if (i == bdec_ZERO)
				dcf77_putBit( 0 );
		}
		/* copy the time to the display */
		sTimeStamp t;
		if (dcf77_getTime( &t ) == 0) {
			led_putDec( 0, t.minute, t.second & 0x01 );
			led_putHex( 2, 0x10, 0 );
			led_putDec( 3, t.hour, 0 );
		} else {
			for (uint8_t j = 0; i<5; i++)
				led_putHex( i, 0x10, (dot == i) );
		}
		/* handle events that occur very rarely: */
		// 1s-timer
		if (Timer1s >= 0) {
			cli();
			Timer1s = Timer1s - 5;
			sei();

			dcf77_timer();
			light_slow_timer();
			light_start();
			dot++;
			if (dot >= 5) dot = 0;
		}
	}

}

ISR(TIMER0_COMPA_vect) {
	if (--Timer200ms == 0) {
		Timer200ms = 125;
		SyncTimer++;
		Timer1s++;
	}
	sei();
	bdec_timer();
	light_timer();
	cli();
}